# -*- coding: utf-8; -*-

# This file is part of python-am2315 library.
#
# python-am2315 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-am2315 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with python-am2315.  If not, see <http://www.gnu.org/licenses/>

# 2017-09-07 David Goncalves <david@lestat.st>
#            Python class to communicate with an AM2315 chip on I2C bus.
#            It's an AOSONG temperature and humidity sensor.

import smbus
import sys
import time


class am2315:

    last_wakeup = 0

    def __init__(self, address=0x5c, bus=1, debug=False):
        self._debug = debug
        self.i2c_addr = address
        self.i2c_bus = smbus.SMBus(bus)

        self._debug_message("Accessing AM2315 at address 0x%x on bus %s" % (address, bus))

    def _debug_message(self, message="", error=False):
        """
        Prints debug messages to console when debuging mode is activated
        """

        if (self._debug is True) or (error is True):
            print("%s %s" % (("[DEBUG]" if error is False else "[ERROR]"), message))
            sys.stdout.flush()
            if (error is True):
                raise Exception(message)

    def _wakeup_sensor(self):
        """
        Sends a dummy command to the sensor to wake-it up. By design this sensor goes
        to powersave mode when there is no activity to avoid self heating and then
        influencing the measurements.
        """

        self._debug_message("Waking-up device...")
        # This command will fail but is just intended to wakeup the device
        try:
            self.i2c_bus.write_i2c_block_data(self.i2c_addr, 0x03, [0x0A, 0x01])
            time.sleep(0.125)
            data = self.i2c_bus.read_i2c_block_data(self.i2c_addr, 0, 5)

        except IOError:
            pass

        self.last_wakeup = time.time()

    def _verify_crc(self, data):
        """
        Compute the CRC of 'data' according to the method on AM2315's datasheet.
        The result is sent as a 2 bytes list with the high byte first to be compatible
        with the chip's communication method.
        """

        crc = 0xFFFF
        for c in data:
            crc = crc ^ c
            for i in range(1, 9):
                if (crc & 0x01):
                    crc = crc >> 1
                    crc = crc ^ 0xA001

                else:
                    crc = crc >> 1

        return [crc & 0x00FF, crc >> 8]

    def _send_data(self, data):
        """
        Sends a command to the device and reads the response from bus. This
        method will also check the response data (verify CRC, etc...)

        The data format is a list containing the i2c command (position 0) and
        the arguments to send to the chip.
        """

        # The sensor cannot be up more than 3s...
        if ((time.time() - self.last_wakeup) >= 3):
            self._wakeup_sensor()

        self._debug_message("Sending data    : %s" % ' '.join(["0x%02X" % x for x in data]))
        self.i2c_bus.write_i2c_block_data(self.i2c_addr, data[0], data[1:])
        time.sleep(0.125)

        if (data[0] == 0x03):
            # If command is '03' (read register) the len is eqal to:
            # cmd + number of registers + register A,B,C,X + crc
            read_len = data[2] + 4

        elif (data[0] == 0x10):
            # If command is '10' (write register) the len is eqal to:
            # cmd + saved data len + crc
            read_len = 4

        else:
            self._debug_message("Unsupported chip command '%h'" % data[0], True)

        resp = self.i2c_bus.read_i2c_block_data(self.i2c_addr, 0, read_len)
        self._debug_message("Sensor response : %s" % ' '.join(["0x%02X" % x for x in resp]))

        crc = self._verify_crc(resp[:-2])
        self._debug_message("Calculated CRC  : %s" % ' '.join(["0x%02X" % x for x in crc]))

        if (crc != resp[-2:]):
            self._debug_message("CRC mismatch... error during transmission", True)

        return(resp)

    def GetTemperature(self):
        """
        Gets the temperature from sensor
        """

        self._debug_message("Reading sensor temperature")
        data = self._send_data([0x03, 0x02, 0x02])

        temp = (((data[2] & 0x7F) << 8) + data[3]) / 10.0
        if (data[2] >> 7) == 1:
            temp = - temp

        self._debug_message("Sensor temperature is %s°C" % temp)
        return(temp)

    def GetHumidity(self):
        """
        Gets the relative humidity from sensor
        """

        self._debug_message("Reading sensor humidity")
        data = self._send_data([0x03, 0x00, 0x02])

        humidity = ((data[2] << 8) + data[3]) / 10.0

        self._debug_message("Sensor mesured humidity is %s%%" % humidity)
        return(humidity)
